import { Hono } from 'hono'
import { zValidator } from '@hono/zod-validator'
import { z } from 'zod'
import { EmailService } from '../services/email'
import { getEmailTemplate } from '../templates'
import { Language } from '../types'
import { toBuffer } from 'bwip-js/node'

const emailRouter = new Hono()
const emailService = new EmailService()

const sendEmailSchema = z.object({
  to: z.string().email(),
  subject: z.string(),
  templateName: z.string(),
  language: z.enum(['en', 'de']),
  data: z.record(z.any())
})

async function generateBarcodeDataURL(data) {
  const buffer = await toBuffer({
    bcid: 'code128',
    text: data,
    scale: 3,
    height: 10,
    includetext: true,
    textxalign: 'center',
  });

  const base64Data = buffer.toString('base64');
  const dataURL = `data:image/png;base64,${base64Data}`;

  return dataURL;
}

emailRouter.post('/', zValidator('json', sendEmailSchema), async (c) => {
  let { to, subject, templateName, language, data } = c.req.valid('json')

  try {
    const template = getEmailTemplate(templateName, language as Language)
    if (templateName === "welcome") {
      const dataURL = await generateBarcodeDataURL(data.barcode_content);
      data.barcode_url = dataURL;
    }
    if (templateName === "password-reset") {
      data.reset_link= `${process.env.FRONTEND_URL}/reset/${(Buffer.from(data.token)).toString("base64")}`
    }
    await emailService.sendEmail({
      to,
      subject,
      html: template.html(data),
      text: template.text(data)
    })

    return c.json({ success: true })
  } catch (error) {
    return c.json({ success: false, error: (error as Error).message }, 400)
  }
})

// Add queue status endpoint
emailRouter.get('/status', async (c) => {
  try {
    const status = await emailService.getQueueStatus()
    return c.json(status)
  } catch (error) {
    return c.json({ error: (error as Error).message }, 500)
  }
})

export { emailRouter }